@@ -4,8 +4,8 @@
  */
 import { getInjectable } from "@ogre-tools/injectable";
 import { computed } from "mobx";
-import type { Route } from "../../../../route-injection-token";
-import { routeInjectionToken } from "../../../../route-injection-token";
+import type { Route } from "../../../../front-end-route-injection-token";
+import { frontEndRouteInjectionToken } from "../../../../front-end-route-injection-token";
 
 export interface HelmChartsPathParameters {
   repo?: string;
@@ -21,7 +21,7 @@ const helmChartsRouteInjectable = getInjectable({
     isEnabled: computed(() => true),
   }),
 
-  injectionToken: routeInjectionToken,
+  injectionToken: frontEndRouteInjectionToken,
 });
 
 export default helmChartsRouteInjectable;
