@@ -3,12 +3,14 @@
  * Licensed under MIT License. See LICENSE in root directory for more information.
  */
 import { getInjectable } from "@ogre-tools/injectable";
-import { appSemVer } from "../../../common/vars";
+import appSemanticVersionInjectable from "../../../common/vars/app-semantic-version.injectable";
 import { isCompatibleBundledExtension } from "./is-compatible-bundled-extension";
 
 const isCompatibleBundledExtensionInjectable = getInjectable({
   id: "is-compatible-bundled-extension",
-  instantiate: () => isCompatibleBundledExtension({ appSemVer }),
+  instantiate: (di) => isCompatibleBundledExtension({
+    appSemVer: di.inject(appSemanticVersionInjectable),
+  }),
 });
 
 export default isCompatibleBundledExtensionInjectable;
