@@ -6,8 +6,8 @@ import { getInjectable } from "@ogre-tools/injectable";
 import { workloadInjectionToken } from "../workload-injection-token";
 import { ResourceNames } from "../../../../utils/rbac";
 import navigateToPodsInjectable from "../../../../../common/front-end-routing/routes/cluster/workloads/deployments/navigate-to-deployments.injectable";
-import namespaceStoreInjectable from "../../../+namespaces/namespace-store/namespace-store.injectable";
-import cronJobsStoreInjectable from "../../../+workloads-cronjobs/cron-jobs-store.injectable";
+import namespaceStoreInjectable from "../../../+namespaces/store.injectable";
+import cronJobsStoreInjectable from "../../../+workloads-cronjobs/store.injectable";
 import { computed } from "mobx";
 
 const cronJobsWorkloadInjectable = getInjectable({
