@@ -5,8 +5,8 @@
 import React from "react";
 import { Link } from "react-router-dom";
 import type { PodVolumeVariants, Pod, SecretReference } from "../../../../../common/k8s-api/endpoints";
-import type { KubeApi } from "../../../../../common/k8s-api/kube-api";
-import type { LocalObjectReference, KubeObject } from "../../../../../common/k8s-api/kube-object";
+import type { KubeApiQueryParams, ResourceDescriptor } from "../../../../../common/k8s-api/kube-api";
+import type { LocalObjectReference } from "../../../../../common/k8s-api/kube-object";
 import { DrawerItem } from "../../../drawer";
 import { getDetailsUrl } from "../../../kube-detail-params";
 
@@ -18,11 +18,15 @@ export interface PodVolumeVariantSpecificProps<Kind extends keyof PodVolumeVaria
 
 export type VolumeVariantComponent<Kind extends keyof PodVolumeVariants> = React.FunctionComponent<PodVolumeVariantSpecificProps<Kind>>;
 
+export interface LocalRefPropsApi {
+  getUrl(desc?: Partial<ResourceDescriptor>, query?: Partial<KubeApiQueryParams>): string;
+}
+
 export interface LocalRefProps {
   pod: Pod;
   title: string;
   kubeRef: LocalObjectReference | SecretReference | undefined;
-  api: KubeApi<KubeObject>;
+  api: LocalRefPropsApi;
 }
 
 export const LocalRef = ({ pod, title, kubeRef: ref, api }: LocalRefProps) => {
